% conduct hypothesis tests of GMM estimation
% load simulation results before running, as below
%load runM10000T500S1

% specification test using mean squared pricing error; essentially the same
% as the bootstrap test in Toda & Walsh (2014)
q95PE_std = quantile(results_std(4,:),0.95); % 95 percentile of standard GMM pricing error
rejectRatePE_stdF = sum(results_stdF(4,:)>q95PE_std)/M; % rejection rate of standard GMM with false returns

q95PE_age = quantile(results_age(4,:),0.95); % 95 percentile of age cohort GMM pricing error
rejectRatePE_ageF = sum(results_ageF(4,:)>q95PE_age)/M; % rejection rate of age cohort GMM with false returns

q95PE_RA = quantile(results_RA(4,:),0.95); % 95 percentile of representative-agent GMM pricing error
rejectRatePE_RAF = sum(results_RAF(4,:)>q95PE_RA)/M; % rejection rate of representative-agent GMM with false returns

% J test for true and false models
% For true model, gives size distortions
% For false model, gives Type II error rate
q95J1_std = quantile(results_std(3,:),0.95); % 95 percentile of standard GMM J1 statistic
rejectRateJ1Sim_stdF = sum(results_stdF(3,:)>q95J1_std)/M; % rejection rate of standard GMM with false returns

q95J1_age = quantile(results_age(3,:),0.95); % 95 percentile of age cohort GMM J1 statistic
rejectRateJ1Sim_ageF = sum(results_ageF(3,:)>q95J1_age)/M; % rejection rate of age cohort GMM with false returns

q95J1_RA = quantile(results_RA(3,:),0.95); % 95 percentile of representative-agent GMM J1 statistic
rejectRateJ1Sim_RAF = sum(results_RAF(3,:)>q95J1_RA)/M; % rejection rate of representative-agent GMM with false returns

K = 1+size(instrument,1); % number of equations
%Ncohort = length(cutoff); % number of cohorts
Ncohort = 1; % use this if it is reasonable to assume that equations for cohorts are almost the same
rejectRateJ1_std = sum(1-chi2cdf(results_std(3,:),K-1)<0.05)/M;
%rejectRateJ2_std = sum(1-chi2cdf(results_std(7,:),K-1)<0.05)/M;
% rejection rate of standard GMM with true asset returns
rejectRateJ1_age = sum(1-chi2cdf(results_age(3,:),K*Ncohort-1)<0.05)/M;
%rejectRateJ2_age = sum(1-chi2cdf(results_age(7,:),K*Ncohort-1)<0.05)/M;
% rejection rate of age cohort GMM with true asset returns
rejectRateJ1_RA = sum(1-chi2cdf(results_RA(3,:),K-1)<0.05)/M;
%rejectRateJ2_RA = sum(1-chi2cdf(results_RA(7,:),K-1)<0.05)/M;
% rejection rate of representative-agent GMM with true asset returns
rejectRateJ1_stdF = sum(1-chi2cdf(results_stdF(3,:),K-1)<0.05)/M;
%rejectRateJ2_stdF = sum(1-chi2cdf(results_stdF(7,:),K-1)<0.05)/M;
% rejection rate of standard GMM with false asset returns
rejectRateJ1_ageF = sum(1-chi2cdf(results_ageF(3,:),K*Ncohort-1)<0.05)/M;
%rejectRateJ2_ageF = sum(1-chi2cdf(results_ageF(7,:),K*Ncohort-1)<0.05)/M;
% rejection rate of age cohort GMM with false asset returns
rejectRateJ1_RAF = sum(1-chi2cdf(results_RAF(3,:),K-1)<0.05)/M;
%rejectRateJ2_RAF = sum(1-chi2cdf(results_RAF(7,:),K-1)<0.05)/M;
% rejection rate of representative-agent GMM with false asset returns

% t-test of gam1 = gamma; gives Type I error rate
TypeIerrT1_std = sum(1-normcdf(abs(results_std(1,:)-gamma)./results_std(2,:))<0.025)/M;
TypeIerrT1_age = sum(1-normcdf(abs(results_age(1,:)-gamma)./results_age(2,:))<0.025)/M;
TypeIerrT1_RA = sum(1-normcdf(abs(results_RA(1,:)-gamma)./results_RA(2,:))<0.025)/M;

% t-test of gam2 = gamma; gives Type I error rate
%TypeIerrT2_std = sum(1-normcdf(abs(results_std(5,:)-gamma)./results_std(6,:))<0.025)/M;
%TypeIerrT2_age = sum(1-normcdf(abs(results_age(5,:)-gamma)./results_age(6,:))<0.025)/M;
%TypeIerrT2_RA = sum(1-normcdf(abs(results_RA(5,:)-gamma)./results_RA(6,:))<0.025)/M;

Gam1 = [results_std(1,:)' results_age(1,:)' results_RA(1,:)'...
    results_stdF(1,:)' results_ageF(1,:)' results_RAF(1,:)']; % gamma1
%Gam2 = [results_std(5,:)' results_age(5,:)' results_RA(5,:)'...
%    results_stdF(5,:)' results_ageF(5,:)' results_RAF(5,:)']; % gamma2
SE1 = [results_std(2,:)' results_age(2,:)' results_RA(2,:)'...
    results_stdF(2,:)' results_ageF(2,:)' results_RAF(2,:)']; % gamma1
ind1 = find(sum(isnan(Gam1),2)==0);
%ind2 = find(sum(isnan(Gam2),2)==0);
Gam1 = Gam1(ind1,:);
SE1 = SE1(ind1,:);
SE1trunc100 = min(SE1,100); % standard error truncated at 100
%Gam2 = Gam2(ind2,:);
bias1 = mean(Gam1)-gamma; % bias of first stage gamma estimate
%bias2 = mean(Gam2)-gamma; % bias of second stage gamma estimate
se1 = mean(SE1); % mean standard error
se1trunc100 = mean(SE1trunc100); % mean standard error truncated at 100
MAE1 = mean(abs(Gam1-gamma)); % mean absolute error
%MAE2 = mean(abs(Gam2-gamma));
RMSE1 = sqrt(mean((Gam1-gamma).^2)); % root mean squared error
%RMSE2 = sqrt(mean((Gam2-gamma).^2));

% count the number of times with multiple troughs
%multTroughs = sum([results_std(9,:)' results_age(9,:)' results_RA(9,:)']>1);

% count the number of times with pricing error less than 0.01, 0.001,
% 0.0001
PE001 = sum([results_stdF(4,:)' results_ageF(4,:)' results_RAF(4,:)']<=1e-2);
PE0001 = sum([results_stdF(4,:)' results_ageF(4,:)' results_RAF(4,:)']<=1e-3);
PE00001 = sum([results_stdF(4,:)' results_ageF(4,:)' results_RAF(4,:)']<=1e-4);

% output results
bias1 % bias of first stage gamma estimate
se1 % mean standard error
se1trunc100 % mean standard error truncated at 100
MAE1 % mean absolute error
RMSE1 % mean squared error
rejectRatePESim_false = [rejectRatePE_stdF rejectRatePE_ageF rejectRatePE_RAF];
rejectRateJ1Sim_false = [rejectRateJ1Sim_stdF rejectRateJ1Sim_ageF rejectRateJ1Sim_RAF];
rejectRateJ1_true = [rejectRateJ1_std rejectRateJ1_age rejectRateJ1_RA];
rejectRateJ1_false = [rejectRateJ1_stdF rejectRateJ1_ageF rejectRateJ1_RAF];
%rejectRateJ2_true = [rejectRateJ2_std rejectRateJ2_age rejectRateJ2_RA];
%rejectRateJ2_false = [rejectRateJ2_stdF rejectRateJ2_ageF rejectRateJ2_RAF];
TypeIerrT1 = [TypeIerrT1_std TypeIerrT1_age TypeIerrT1_RA]
%TypeIerrT2 = [TypeIerrT2_std TypeIerrT2_age TypeIerrT2_RA]; % second stage poor, ignore
TypeIerrJ1 = rejectRateJ1_true
TypeIIerrPESim = 1-rejectRatePESim_false
TypeIIerrJ1Sim = 1-rejectRateJ1Sim_false
TypeIIerrJ1 = 1-rejectRateJ1_false
%TypeIIerrJ2 = 1-rejectRateJ2_false; % second stage poor, ignore