function [obj,gradObj] = GMMcrit4GEL(gamma,logrelC,age,logC,ER,instrument,cutoff,lags)
% GEL-GMM criterion based on Kitamura & Stutzer (1997)
% lag: number of lags to compute Newey-West (corresponds to K in KS1997)
%T = size(logrelC,2)-1; % sample size in GMM
[g,dg] = residual4(gamma,logrelC,age,logC,ER,instrument,cutoff);

temp1 = tsmovavg(g,'s',2*lags+1,2);
fHat = temp1(:,2*lags+1:end); % fHat in Eq (8) of KS1997
temp2 = tsmovavg(dg,'s',2*lags+1,2);
dfHat = temp2(:,2*lags+1:end); % derivative of fHat
[L,T] = size(fHat);

options = optimset('TolFun',1e-6,'TolX',1e-6,'Display','off','GradObj','on','Hessian','on');

[lambda,fval] = fminunc(@(lambda) GELObjective(lambda,fHat),zeros(L,1),options);

obj = -fval; % GMM criterion
gradObj = -sum(bsxfun(@times,exp(lambda'*fHat),lambda'*dfHat))/T;
end

