% please double check:
% T (sample size, must be 100, 300, or 500)
% M (number of Monte Carlo replications, 10000 for paper)
% unc (whether to estimate an unconditional model (0), conditional model
% (1), or exactly identified model (2)
% file name to save
% before running!

% simulate cCAPM economy with idiosyncratic risk
shillerVAR % estimate VAR for consumption/dividend growth from Shiller data
paramSet % set parameters for simulation
T = 500; % sample size
%sigma_e = 0; % no measurement error
M = 1; % monte carlo replications
unc = 0;
% 0: conditional (use instruments)
% 1: unconditional (no instrument but two assets)
% 2: exactly identified (one asset, no instruments)

% parameters for GMM estimation
grid = 0:1:20; % grid to do grid search for initial value of gamma
% about half of run time is grid search, so decrease number of grid points
% if you are impatient
lags=4; % Newey-West lags
w_false = 1; % weight of the false returns (between 0 and 1)
Ncut = 5; % number of cutoffs;
cutoff = -1/delta*log(1-(1:Ncut)/Ncut); % use this for all cohorts
% age cutoff, defined by theoretical quantiles.  Use Inf if no age cohort
%cutoff = -1/delta*log(1-[1:Ncut-1]/Ncut); % use this for all but oldest cohort

% do GMM
results_std=zeros(5,M);    % results for standard GMM
results_age=zeros(5,M);    % results for age cohort GMM
results_RA=zeros(5,M);     % results for representative-agent GMM
results_stdF=zeros(5,M);   % results for standard GMM with false asset returns
results_ageF=zeros(5,M);   % results for age cohort GMM with false asset returns
results_RAF=zeros(5,M);    % results for representative-agent GMM with false asset returns
results_stdGEL = zeros(5,M);
%results_ageGEL = zeros(5,M);
results_RAGEL = zeros(5,M);
results_stdGELF = zeros(5,M);
%results_ageGELF = zeros(5,M);
results_RAGELF = zeros(5,M);

h = waitbar(0,'Please wait...');
%tic
%parfor_progress(M);
%parfor m=1:M
for m=1:M
    waitbar(m/M);
    fprintf('Current run is %d out of %d \n',m,M) % current Monte Carlo run
    disp('Simulating')
    [logc_obs,age,logC,Rc,Rd,Rf,PCratio,PDratio]...
    = cCAPM_Sim3(I,T,b,beta,gamma,delta,g,A,Psi,sigma_i,sigma_0,sigma_e,N);
    if unc == 0
        Rs = Rd;
        instrument = PDratio./mean(PDratio); % use normalized price-dividend ratio as instrument
    elseif unc == 1
        Rs = [Rc;Rd];
        instrument = [];
    else
        Rs = Rd;
        instrument = [];
    end
    %ER = bsxfun(@minus,Rs(:,2:end),Rf(1:end-1));
    
    % First, do GMM with true asset returns
    % standard GMM
    disp('Standard GMM')
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4_stage1(gamma,logc_obs,age,logC,Rs,Rf,instrument,Inf,grid,lags);
    results_std(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_std(:,m)=NaN*ones(5,1);
    end
    %toc
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4GEL(gamma,logc_obs,age,logC,Rs,Rf,instrument,Inf,grid,lags);
    results_stdGEL(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_stdGEL(:,m)=NaN*ones(5,1);
    end
    %toc
    % age cohort GMM
    disp('Age cohort GMM')
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4_stage1(gamma,logc_obs,age,logC,Rs,Rf,instrument,cutoff,grid,lags);
    results_age(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_age(:,m)=NaN*ones(5,1);
    end
    %toc
    % recommend not to do GEL with age cohort because too time consuming
    %tic
    %try
    %[gam1,se1,J1,meanPE1,ntrough]...
    %    = GMM4GEL(gamma,logc_obs,age,logC,Rs,Rf,instrument,cutoff,grid,lags);
    %results_ageGEL(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    %catch
    %    results_ageGEL(:,m)=NaN*ones(5,1);
    %end
    %toc
    % representative-agent GMM
    disp('Representative-agent GMM')
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4_stage1(gamma,logC,[],logC,Rs,Rf,instrument,Inf,grid,lags);
    results_RA(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_RA(:,m)=NaN*ones(5,1);
    end
    %toc
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4GEL(gamma,logC,[],logC,Rs,Rf,instrument,Inf,grid,lags);
    results_RAGEL(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_RAGEL(:,m)=NaN*ones(5,1);
    end
    %toc
    
    % Next, do GMM with false asset returns
    disp('Using false asset returns')
    R=[Rs;Rf]; % true asset returns
    ind = randperm(T); % random permutation of time
    RF = R(:,ind); % random returns
    Rs = (1-w_false)*Rs+w_false*RF(1:end-1,:); % stock return
    Rf = (1-w_false)*Rf+w_false*RF(end,:); % bond return
    % standard GMM
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4_stage1(gamma,logc_obs,age,logC,Rs,Rf,instrument,Inf,grid,lags);
    results_stdF(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_stdF(:,m)=NaN*ones(5,1);
    end
    %toc
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4GEL(gamma,logc_obs,age,logC,Rs,Rf,instrument,Inf,grid,lags);
    results_stdGELF(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_stdGELF(:,m)=NaN*ones(5,1);
    end
    %toc
    % age cohort GMM
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4_stage1(gamma,logc_obs,age,logC,Rs,Rf,instrument,cutoff,grid,lags);
    results_ageF(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_ageF(:,m)=NaN*ones(5,1);
    end
    %toc
    % recommend not to do GEL with age cohort because too time consuming
    %tic
    %try
    %[gam1,se1,J1,meanPE1,ntrough]...
    %    = GMM4GEL(gamma,logc_obs,age,logC,Rs,Rf,instrument,cutoff,grid,lags);
    %results_ageGELF(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    %catch
    %    results_ageGELF(:,m)=NaN*ones(5,1);
    %end
    %toc
    % representative-agent GMM
    %tic
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4_stage1(gamma,logC,[],logC,Rs,Rf,instrument,Inf,grid,lags);
    results_RAF(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_RAF(:,m)=NaN*ones(5,1);
    end
    %toc
    try
    [gam1,se1,J1,meanPE1,ntrough]...
        = GMM4GEL(gamma,logC,[],logC,Rs,Rf,instrument,Inf,grid,lags);
    results_RAGELF(:,m)=[gam1,se1,J1,meanPE1,ntrough]';
    catch
        results_RAGELF(:,m)=NaN*ones(5,1);
    end
    %toc
    %parfor_progress;
end
close(h)
%parfor_progress(0);
%toc

% uncomment below if you want to plot results
% GMM_figs

% uncomment below if you want to do hypothesis testing
% GMM_stats

% If you want to save results automatically, uncomment below and change
% file name to different one.
%** please use runM-----T---unc for unconditional, where - is sample size
%** please use runM-----T---cond for conditional, where - is sample size
%save falename
