close all
% Change default axes fonts
%set(0,'DefaultAxesFontName', 'Times New Roman')
set(0,'DefaultAxesFontSize', 14)

% Change default text fonts
%set(0,'DefaultTextFontname', 'Times New Roman')
set(0,'DefaultTextFontSize', 14)
set(0,'DefaultLineLineWidth',2)

% simulate cCAPM economy with idiosyncratic risk
shillerVAR % estimate VAR for consumption/dividend growth from Shiller data
paramSet % set parameters for simulation

% parameters for GMM estimation
grid = 0:1:20; % grid to do grid search for initial value of gamma
N = length(grid);
lags=4; % Newey-West lags

% do standard GMM, stop when we get 2 troughs with a spurious gamma
spurious = 0;
m = 0;
while spurious==0
    m = m+1;
    fprintf('Current run is %d \n',m) % current Monte Carlo run
    tic
    % simulate economy
    [logc_obs,age,logC,Rs,Rf,PDratio,alpha,X] = cCAPM_Sim3(I,T,b,beta,gamma,delta,g,A,Psi,sigma_i,sigma_0,sigma_e,N);
    instrument = PDratio./mean(PDratio); % use normalized price-dividend ratio as instrument
    % standard GMM with true assets
    logrelC = logc_obs-repmat(logC,I,1); % log relative consumption
    Rs(1)=[]; % stock return from t = 2 to T
    Rf(end)=[]; % risk-free rate from t = 1 to T-1
    EP = Rs-Rf; % equity premium
    clear logc_obs Rs Rf
    K = size(instrument,1)+1; % number of equations
    options = optimset('GradObj','on','Display','off','Algorithm','interior-point');
    W = eye(K); % weighting matrix
    obj = @(gamma)(GMMcrit4(gamma,logrelC,age,logC,EP,instrument,Inf,W));
    % first, do a grid search on a coarse grid to get initial value
    crit=zeros(1,N); % GMM criterioin at grid points
    for n=1:N
        crit(n) = GMMcrit4(grid(n),logrelC,age,logC,EP,instrument,Inf,W);
    end
    critSign = sign(diff(crit)); % sign of difference of criterion; 1 if increasing, -1 if decreasing
    trough = critSign(2:end)>critSign(1:end-1);
    ntrough = sum(trough); % number of troughs in grid search
    if ntrough>1
        trough_ind = find(trough)-1; % index of trough
        Gamma = grid(trough_ind);
        [gam1,fval1] = fminbnd(obj,max(Gamma(1)-1,0),Gamma(1)+1);
        [gam2,fval2] = fminbnd(obj,max(Gamma(2)-1,0),Gamma(2)+1);
        % find local minima of GMM criterion
        spurious = (fval2<fval1);
    end
    toc
end
disp('You got 2 troughs with spurious gamma!');

% plot GMM criterion
%gammax = ceil(gam1)+1;
gammax = max(grid);
gamgrid1 = 0:0.1:gammax; % coarse grid
gamgrid2 = gam2-0.1:0.01:gam2+0.1; % fine grid around gamma estimate
gamgrid = union(gamgrid1,gamgrid2);
Ngam = length(gamgrid);
crit = zeros(1,Ngam);
for n=1:Ngam
    crit(n) = GMMcrit4(gamgrid(n),logrelC,age,logC,EP,instrument,Inf,W);
end
PE = sqrt(crit/(2*T));
figure
plot(gamgrid,PE,'k')
xlabel('RRA Coefficient')
ylabel('Pricing Error')
xlim([0 gammax])
ylim([0 0.1])
