function [obj,gradObj] = perror4(gamma,logrelC,age,logC,ER,instrument,cutoff)
% compute pricing error and its gradient

% logrelC: log relative consumption
% age: age of agents
% logC: log aggregate consumption
% ER: matrix of excess returns
% instrument: instrument
% cutoff: age cutoffs (vector of upper bounds of each cohort)

T=size(logrelC,2); % number of time periods
Z=[ones(1,T);instrument]; % instruments
Z(:,end)=[]; % delete last observation
K=size(Z,1); % number of equations
N=size(ER,1); % number of assets
A = length(cutoff); % number of age cohorts
bound = [0, cutoff];

logC1 = logC(2:end);
logC0 = logC(1:end-1);
Cgrowthgam = exp(-gamma*(logC1-logC0)); % -gamma th power of aggregate consumption growth
relCgam = exp(-gamma*logrelC); % -gamma th power of relative consumption
repZ = kron(Z,ones(N,1)); % replicate Z by the number of assets

if A==1
    clear age
    relCm = mean(relCgam,1);
    % sample cross-sectional moment of relative consumption
    SDF = Cgrowthgam.*(relCm(2:end)./relCm(1:end-1));
    obj = mean(repmat(bsxfun(@times,SDF,ER),K,1).*repZ,2);
    temp = mean(relCgam.*logrelC,1)./relCm;
    clear relCm relCgam logrelC fT
    % use this to compute derivative
    dSDF = SDF.*(-temp(2:end)+temp(1:end-1)-logC1+logC0);
    clear temp
    % derivative of SDF with respect to gamma
    gradObj=mean(repmat(bsxfun(@times,dSDF,ER),K,1).*repZ,2);
else GT = zeros(K*N,A);
    dGT = zeros(K*N,A);
    for a=1:A
    incl = (age>=bound(a)).*(age<bound(a+1));
    % inclusion criterion of agents (current period)
    Na = sum(incl); % number of agents in cohort at each time
    temp1 = relCgam.*incl;
    clear incl
    relCm = sum(temp1)./Na; % cross-sectional moment (current period)
    incl_next = (age>=bound(a)+1).*(age<bound(a+1)+1);
    % inclusion criterion of agents (next period): age increases by 1, so
    % must add 1 to bounds
    Na_next = sum(incl_next);
    temp1_next = relCgam.*incl_next;
    clear incl_next
    relCm_next = sum(temp1_next)./Na_next; % cross-sectional moment (next period)
    SDFa = Cgrowthgam.*(relCm_next(2:end)./relCm(1:end-1));
    % SDF of cohort
    temp = (sum(temp1.*logrelC)./Na)./relCm;
    clear relCm temp1 Na
    temp_next = (sum(temp1_next.*logrelC)./Na_next)./relCm_next;
    % use this to compute derivative
    clear relCm_next temp1_next Na_next
    dSDFa = SDFa.*(-temp_next(2:end)+temp(1:end-1)-logC1+logC0);
    clear temp temp_next
    GT(:,a) = mean(repmat(bsxfun(@times,SDFa,ER),K,1).*repZ,2); % pricing error
    dGT(:,a) = mean(repmat(bsxfun(@times,dSDFa,ER),K,1).*repZ,2); % derivative of SDF with respect to gamma
    end
    obj = reshape(GT,K*N*A,1); gradObj = reshape(dGT,K*N*A,1); % vectorize results
end
end