close all
% Change default axes fonts
%set(0,'DefaultAxesFontName', 'Times New Roman')
set(0,'DefaultAxesFontSize', 14)

% Change default text fonts
%set(0,'DefaultTextFontname', 'Times New Roman')
set(0,'DefaultTextFontSize', 14)
set(0,'DefaultLineLineWidth',2)

colors = get(gca,'ColorOrder');
c1 = colors(1,:);
c2 = colors(2,:);

% simulate cCAPM economy with idiosyncratic risk
shillerVAR % estimate VAR for consumption/dividend growth from Shiller data
paramSet % set parameters for simulation

dividendLeverage = 1;

[logc_obs,age,logC,Rc,Rd,Rf,PCratio,PDratio,alpha,X]...
    = cCAPM_Sim3(I,T,b,beta,gamma,delta,g,A,Psi,sigma_i,sigma_0,sigma_e,N,dividendLeverage);
c_ub = 2; % upper bound for plotting

logcgrid = -c_ub:0.01:c_ub;

figure(1)
x = logc_obs(:,end)-logC(end);
histogram(x,'Normalization','pdf'); hold on
plot(logcgrid,NL_pdf(logcgrid,[-sigma_0^2/2-sigma_e^2/2,sqrt(sigma_0^2+sigma_e^2),alpha])); hold off
xlim([-c_ub c_ub]);
xlabel('Log Relative Consumption')
ylabel('Probability Density')
legend('Histogram','Theoretical')

% QQ plot for normal density
m = mean(x);
s = std(x,1);
normsim=normrnd(m,s,[I,1]);
figure(2)
h = qqplot(normsim,x);
set(h(1),'markeredgecolor',c1);
set(h(2),'color',c2);
set(h(3),'color',c2);
xlim([-c_ub,c_ub])
ylim([-c_ub,c_ub])
xlabel('Quantiles of fitted normal')
ylabel('Quantiles of data')
% QQ plot for normal-Laplace density
NLsim=NLrnd([-sigma_0^2/2-sigma_e^2/2,sqrt(sigma_0^2+sigma_e^2),alpha],[I,1]);
figure(3)
h = qqplot(NLsim,x);
set(h(1),'markeredgecolor',c1);
set(h(2),'color',c2);
set(h(3),'color',c2);
xlim([-c_ub,c_ub])
ylim([-c_ub,c_ub])
xlabel('Quantiles of theoretical normal-Laplace')
ylabel('Quantiles of data')

figure(4)

histogram(age(:,end),'Normalization','pdf'); hold on
plot(1:T,delta*exp(-delta*(1:T))); hold off
xlim([0 floor(5/delta)])
xlabel('Age')
ylabel('Probability Density')
legend('Histogram','Theoretical')

time = 1:121;

figure(5)
plot(time+1888,PD_data,time+1888,PDratio(time))
xlabel('Year')
ylabel('Price-Dividend Ratio')
xlim([1889 2009])
legend('Actual','Simulated')

figure(6)
plot(time+1888,PCratio(time))
xlabel('Year')
ylabel('Price-Consumption Ratio')
xlim([1889 2009])
legend('Simulated')

figure(7)
plot(time+1888,100*(Rd(time)-1),'-'); hold on
plot(time+1888,100*(Rf(time)-1),'-.'); hold off
%plot(time+1888,100*(Rc(time)-1))
xlabel('Year')
xlim([1889 2009])
ylabel('Asset Returns (%)')
legend('Stock','Bond')
%legend('Dividend Claim','Bond','Consumption Claim')

figure(8)
plot(time+1888,100*(Rs_data-1),'-'); hold on
plot(time+1888,100*(Rf_data-1),'-.'); hold off
xlabel('Year')
xlim([1889 2009])
ylabel('Asset Returns (%)')
legend('Stock','Bond')

tic
Nm = 9; % number of VAR states in each dimension
S = Nm^2;
a = (eye(2)-A)*g; % constant term of VAR
[P,x] = discreteVAR(a,A,Psi,Nm,2,'even'); % discretize VAR
vc = [1-gamma,0]';
vd = [-gamma,1]';
tempC = beta*(1-delta)*exp(1/2*gamma*(gamma+1)*sigma_i^2)*P*diag(exp(x'*vc));
tempD = beta*(1-delta)*exp(1/2*gamma*(gamma+1)*sigma_i^2)*P*diag(exp(x'*vd));
PCx = (eye(S)-tempC)\(tempC*ones(S,1)); % vector of price-consumption ratio in each state
PDx = (eye(S)-tempD)\(tempD*ones(S,1)); % vector of price-dividend ratio in each state
[eigvec,eigval] = eigs(P',1); % eigenvector of P'
pi = eigvec/sum(eigvec); % stationary distribution
Rcxx = repmat((PCx'+1).*exp(x(2,:)),S,1)./repmat(PCx,1,S); % matrix of consumption claim returns
Rdxx = repmat((PDx'+1).*exp(dividendLeverage*x(2,:)),S,1)./repmat(PDx,1,S); % matrix of dividend claim returns
ERcx = sum(P.*Rcxx,2); % vector of expected consumption claim return in each state
ERdx = sum(P.*Rdxx,2); % vector of expected dividend claim return in each state
ERc = pi'*ERcx % unconditional expected consumption claim return
ERd = pi'*ERdx % unconditional expected dividend claim return
vf = [-gamma,0]';
G = repmat(g,1,S);
Rfx = 1./(beta*(1-delta)*exp(1/2*gamma*(gamma+1)*sigma_i^2+(G+A*(x-G))'*vf+1/2*vf'*Psi*vf));
% vector of risk-free rate in each state
ERf = pi'*Rfx % unconditional expected risk-free rate
volRc = sqrt(pi'*sum(P.*(Rcxx-ERc).^2,2)); % unconditional volatility of consumption claim
volRd = sqrt(pi'*sum(P.*(Rdxx-ERd).^2,2)); % unconditional volatility of dividend claim
volRf = sqrt(pi'*(Rfx-ERf).^2); % unconditional volatility of risk-free rate
toc

Cg = diff(logC,1);
correlation = corr([Cg;Rc(2:end);Rd(2:end)]')
meanReturns = mean([Rc;Rd]')

logRd5 = tsmovavg(log(Rd),'s',5);
logRf5 = tsmovavg(log(Rf),'s',5);
EP5 = exp(logRd5)-exp(logRf5);

figure(9)
plot(PDratio(1:end-5),EP5(6:end),'.')